# Copyright 2014 Google Inc. All rights reserved.
#
# Modifications Copyright (C) 2018 HeiGIT, University of Heidelberg.
#
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
#

"""Tests for the convert module."""

import unittest

from openrouteservice import convert


class ConvertTest(unittest.TestCase):

    def test_single_coord_tuple(self):
        expected = "1,2"
        ll = (1, 2)
        self.assertEqual(expected, convert._build_coords(ll))

        ll = [1, 2]
        self.assertEqual(expected, convert._build_coords(ll))

        with self.assertRaises(TypeError):
            convert._build_coords(1)

        with self.assertRaises(TypeError):
            convert._build_coords({'lat':1, 'lon':2})
            

        with self.assertRaises(TypeError):
            convert._build_coords('1,2')
            
            
    def test_multi_coord_tuple(self):
        expected = "1,2|3,4"

        ll = ((1,2),(3,4))
        self.assertEqual(expected, convert._build_coords(ll))

        ll = [(1,2),(3,4)]
        self.assertEqual(expected, convert._build_coords(ll))
        
        ll = ([1,2],[3,4])
        self.assertEqual(expected, convert._build_coords(ll))
        
        ll = [[1,2],[3,4]]
        self.assertEqual(expected, convert._build_coords(ll))
        
        with self.assertRaises(TypeError):
            convert._build_coords({{'lat':1, 'lon':2},{'lat':3, 'lon':4}})
            

        with self.assertRaises(TypeError):
            convert._build_coords('[1,2],[3,4]')
            
    def test_polyline_decode(self):
        syd_mel_route = (r"mtkeHuv|q@~@VLHz@\PR|@hBt@j@^n@L\NjALv@Jh@NXi@zBm@jC"
                         "KTy@z@qAhBa@\[Ne@DgCc@i@?[Ty@hAi@zASRi@R}@H_@N[b@kAd"
                         "Cy@`Au@d@eA|@q@h@WRe@PYHYBqADgAAcAL_A^w@~@q@`@w@Zw@C"
                         "m@K[PeA|Aa@p@g@fAiAhBuAv@]VU^k@xAUXe@TqATy@V}@f@_@VO"
                         "\Mb@[fBe@|@Mp@WbCgClKSdAq@Rm@?g@WYg@G[[}Bk@qBy@wDUm@"
                         "w@}@q@}A]o@k@y@kAjC_AjC_ApCe@z@i@j@q@f@[NsAp@u@T}A\w"
                         "ATU?WCeBm@q@MwAGUCg@SMaAi@mDQm@K}@Mq@u@mAc@i@c@Ys@[W"
                         "W_@q@e@a@cA_@w@E{BHmBXqBkBsA}@{Ao@iAB{@QYSi@qCUy@Ee@"
                         "@SDWbA_BLKLAVNb@r@J@HEHK?]k@iDe@w@COAWBUh@qBDc@?c@Q{"
                         "BGa@MQKCOBgA\{@AKEs@Wq@i@q@{@s@gAk@kA]g@g@_@I]??k@i@"
                         "yBkEa@}@W}@WkCUqC?_@Hg@ZqABg@Gm@YoAEgAMq@@jAB|CC`@{@rACH")

        points = convert.decode_polyline(syd_mel_route)['coordinates']
        self.assertAlmostEqual(8.344268, points[0][0], places=5)
        self.assertAlmostEqual(48.233826, points[0][1], places=5)
        self.assertAlmostEqual(8.343433, points[-1][0], places=5)
        self.assertAlmostEqual(48.263552, points[-1][1], places=5)



# Copyright 2014 Google Inc. All rights reserved.
#
# Modifications Copyright (C) 2018 HeiGIT, University of Heidelberg.
#
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
#

"""Performs requests to the ORS geocoding API."""
from openrouteservice import convert


def geocode(client, query,
            lang=None,
            boundary_type=None,
            rect=None,
            circle=None,
            limit=None):
    """
    Geocoding is the process of converting addresses into geographic
    coordinates.

    :param query: Name of location, street address or postal code. For a 
        structured geocoding request, a dict object can be passed. Please refer
        to https://github.com/GIScience/openrouteservice-docs#geocoding-structured-query
        for details. 
    :type query: string or dict of structured geocoding request

    :param lang: Specifies the language of the response. One of ['de', 'en',
        'fr', 'it']. Default 'en'.
    :type lang: string

    :param boundary_type: Specifies the type of spatial search restriction. 
        One of ['rect','circle'].
    :type boundary_type: string

    :param rect: For boundary_type=rect only! Sets the restriction rectangle's
        minimum/maximum longitude/latitude, i.e. [MinLong,MinLat,MaxLong,Maxlat].
    :type rect: list of integers

    :param circle: For boundary_type=circle only! Sets the restriction circle 
        with a center point and a radius in meters, i.e. [Long,Lat,Radius]. 
    :type circle: list of integers

    :param limit: Specifies the maximum number of responses. Default 5.
    :type limit: integer
    
    :raises ValueError: When parameter has invalid value(s).

    :rtype: dict from JSON response
    """

    params = dict()

    if query:
        # Is not checked on backend
        if isinstance(query, dict):
            allowed_keys = ['address',
                            'neighbourhood',
                            'borough',
                            'locality',
                            'county',
                            'region',
                            'postalcode',
                            'country']
            if not all((key in allowed_keys) for key in query.keys()):
                raise ValueError("Geocoding query contains invalid admin parameter.")
        params["query"] = str(query)

    if lang:
        # Is not checked on backend
        if lang not in ['en', 'fr', 'de', 'it']:
            raise ValueError("Invalid language {} specified)".format(lang))
        params["lang"] = lang

    if boundary_type:
        params["boundary_type"] = boundary_type

    if rect:
        params["rect"] = convert._comma_list(rect)

    if circle:
        params["circle"] = convert._comma_list(circle)

    if limit:
        params["limit"] = str(limit)

    return client.request("/geocoding", params)


def reverse_geocode(client, location,
                    lang=None,
                    boundary_type=None,
                    rect=None,
                    circle=None,
                    limit=None):
    """
    Reverse geocoding is the process of converting geographic coordinates into a
    human-readable address.

    :param location: Coordinate to be inquired.
    :type location: lng/lat pair as list or tuple

    :param lang: Specifies the language of the response. One of ['de', 'en',
        'fr', 'it']. Default 'en'.
    :type lang: string

    :param boundary_type: Specifies the type of spatial search restriction. 
        One of ['rect','circle'].
    :type boundary_type: string

    :param rect: For boundary_type=rect only! Sets the restriction rectangle's
        minimum/maximum longitude/latitude, i.e. [MinLong,MinLat,MaxLong,Maxlat].
    :type rect: list of integers

    :param circle: For boundary_type=circle only! Sets the restriction circle 
        with a center point and a radius in meters, i.e. [Long,Lat,Radius]. 
    :type circle: list of integers

    :param limit: Specifies the maximum number of responses. Default 5.
    :type limit: integer
    
    :raises ValueError: When parameter has invalid value(s).

    :rtype: dict from JSON response
    """
    
    params = dict()

    # Check if latlng param is a place_id string.
    #  place_id strings do not contain commas; latlng strings do.
    if location:
        params["location"] = convert._comma_list(location)

    if lang:
        # Is not checked on backend
        if lang not in ['en', 'fr', 'de', 'it']:
            raise ValueError("Invalid language {} specified)".format(lang))
        params["lang"] = lang

    if boundary_type:
        params["boundary_type"] = boundary_type

    if rect:
        params["rect"] = convert._comma_list(rect)

    if circle:
        params["circle"] = convert._comma_list(circle)

    if limit:
        params["limit"] = str(limit)

    return client.request("/geocoding", params)



# Copyright 2014 Google Inc. All rights reserved.
#
# Modifications Copyright (C) 2018 HeiGIT, University of Heidelberg.
#
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy of
# the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations under
# the License.
#

"""Tests for the geocoding module."""

import responses

import test as _test
import unittest
import openrouteservice
from collections import OrderedDict

class GeocodingTest(_test.TestCase):
    
    def setUp(self):
        self.key = '58d904a497c67e00015b45fcacecf32dfe6248f4bd208bc3dc37e113'
        self.client = openrouteservice.Client(self.key)
        self.query = 'Heidelberg'
        self.location = (8.68353,49.412623)
        self.structured = {"postalcode": "69120",
                            "country":"Germany",
                            "locality": "Heidelberg",
                            }

    @responses.activate
    def test_simple_geocode(self):
        responses.add(responses.GET,
                      'https://api.openrouteservice.org/geocoding',
                      body='{"status":"OK","results":[]}',
                      status=200,
                      content_type='application/json')

        results = self.client.geocode(self.query)

        self.assertEqual(1, len(responses.calls))
        self.assertURLEqual('https://api.openrouteservice.org/geocoding?'
                            'api_key={}&query=Heidelberg'.format(self.key),
                            responses.calls[0].request.url)
#
    @responses.activate
    def test_reverse_geocode(self):
        responses.add(responses.GET,
                      'https://api.openrouteservice.org/geocoding',
                      body='{"status":"OK","results":[]}',
                      status=200,
                      content_type='application/json')

        results = self.client.reverse_geocode(self.location)

        self.assertEqual(1, len(responses.calls))
        self.assertURLEqual('https://api.openrouteservice.org/geocoding?'
                            'api_key={}&location=8.68353%2C49.412623'.format(self.key),
                            responses.calls[0].request.url)

    def test_geocode_structured_query(self):
        with self.assertRaises(ValueError):
            self.structured['Apartment'] = 'Below Bridge No 5'
            self.client.geocode(self.structured)
